/* TM1637_clock2
 *
 * Egyszerű óra egy DS3231 Real Time óra és egy 
 * TM1637 négyjegyű 7-szegmens kijelző felhasználásával. 
 * Az óra beállítása két nyomógombbal történik:   
 * a G1 gomb az órákat, a G2 gomb a perceket lépteti   
 * 
 * A programhoz felhasználtuk az Avishay Orpaz által   
 * írt TM1637Display programkönyvtárat:   
 * https://github.com/avishorp/TM1637 
 * valamint Andrew Wickert DS3231 programkönyvtárát:
 * https://github.com/NorthernWidget/DS3231
 */
#include <TM1637Display.h>
#include <DS3231.h>
#include <Wire.h>
#define CLKPIN  9                     // TM1637 CLK láb D9-re
#define DIOPIN  8                     // TM1637 DIO láb D8-ra
#define G1      7                     // Button 1
#define G2     10                     // Button 2 
#define SQW     2                     // Megszakítás

DS3231 Clock;                         // Az RTC példányosítása
TM1637Display display(CLKPIN,DIOPIN); // 4-számjegyű kijelző példányosítása

bool h12, PM;                         // 12/24 mód és AM/PM jelzőbitek
int hh = 0;                           // órák száma
int mm = 0;                           // percek száma
int dp = 0;                           // Kettőspont kijezés (0 vagy 64)
int d2new = 0;                        // D2 bemenet új állapota
int d2old = 0;                        // D2 bemenet előző állapota
int dflag = 0;                        // nem nulla, ha frissíteni kell a kijelzést

void setup() {
  pinMode(G1, INPUT_PULLUP);
  pinMode(G2, INPUT_PULLUP);
  pinMode(SQW, INPUT_PULLUP);
  Wire.begin();                       // Az I2C illesztő inicializálása
  display.setBrightness(0x0F);        // a kijelző bekapcsolása max. fényerőn
  Clock.enableOscillator(true, 0, 0);
}

void loop() {
  //-- A D2-re kötött négyszögjel vizsgálata ----
  d2new = digitalRead(SQW);
  if (d2new != d2old) {               // D2 állapot beolvasása
    dp ^= 64;                         // dp = 0 vagy 64
    dflag++;                          // frissítés kijelzés, ha változás van
  }
  //-- A G1 gomb lenyomásának vizsgálata ----  
  d2old = d2new;
  if ( digitalRead(G1) == LOW) {      // Ha a G1 gomb le van nyomva...
    hh = (hh + 1) % 24;               // Az órák léptetése
    Clock.setHour(hh);
    dflag++;                          // Képfrissítés kell
    delay(100);
  }
  //-- A G2 gomb lenyomásának vizsgálata ----  
  if ( digitalRead(G2) == LOW) {      // Ha a G2 gomb le van nyomva...
    mm = (mm + 1) % 60;               // A percek léptetése
    Clock.setMinute(mm);
    dflag++;                          // Képfrissítés kell
    delay(100);
  }
  //-- A kijelzés frissítése, ha szükséges ----  
  if (dflag) {                        // Ha frissíteni kell
    hh = Clock.getHour(h12, PM);
    // if (h12&&PM) hh=(hh + 12)%24;  // 12 -> 24 konverzió
    mm = Clock.getMinute();           // Percek kiolcvasása
    display.showNumberDecEx(hh,dp,false,2,0); // Óra kijelzése kettősponttal
    display.showNumberDec(mm, true,2,2);      // percek kijelzése
    dflag = 0;
  }
  delay(100);
}
